@@ -6,6 +6,7 @@
     SPDX-FileCopyrightText: 2010 Teo Mrnjavac <teo@kde.org>
     SPDX-FileCopyrightText: 2013 David Faure <faure+bluesystems@kde.org>
     SPDX-FileCopyrightText: 2017 Harald Sitter <sitter@kde.org>
+    SPDX-FileCopyrightText: 2021 Julius Künzel <jk.kdedev@smartlab.uber.space>
 
     SPDX-License-Identifier: LGPL-2.0-or-later
 */
@@ -13,10 +14,11 @@
 #ifndef KABOUTDATA_H
 #define KABOUTDATA_H
 
-#include <kcoreaddons_export.h>
-#include <QString>
 #include <QSharedDataPointer>
+#include <QString>
 #include <QVariant>
+#include <kcoreaddons_export.h>
+#include <memory>
 #include <qcontainerfwd.h>
 
 class QCommandLineParser;
@@ -25,7 +27,11 @@ class KAboutData;
 class KPluginMetaData;
 namespace KCrash
 {
+#ifdef KCOREADDONS_STATIC
+void defaultCrashHandler(int sig);
+#else
 Q_DECL_IMPORT void defaultCrashHandler(int sig);
+#endif
 }
 
 /**
@@ -61,6 +67,8 @@ class KCOREADDONS_EXPORT KAboutPerson
     Q_PROPERTY(QString webAddress READ webAddress CONSTANT)
     Q_PROPERTY(QString ocsUsername READ ocsUsername CONSTANT)
     friend class KAboutData;
+    friend class KAboutDataPrivate;
+
 public:
     /**
      * Convenience constructor
@@ -155,8 +163,8 @@ private:
      */
     explicit KAboutPerson(const QString &name, const QString &email, bool disambiguation);
 
-    class Private;
-    Private *const d;
+private:
+    QSharedDataPointer<class KAboutPersonPrivate> d;
 };
 
 /**
@@ -177,26 +185,27 @@ class KCOREADDONS_EXPORT KAboutLicense
     Q_PROPERTY(KAboutLicense::LicenseKey key READ key CONSTANT)
     Q_PROPERTY(QString spdx READ spdx CONSTANT)
     friend class KAboutData;
-public:
+    friend class KAboutComponent;
 
+public:
     /**
-     * Describes the license of the software.
+     * Describes the license of the software; for more information see: https://spdx.org/licenses/
      */
     enum LicenseKey {
-        Custom = -2,
-        File = -1,
-        Unknown = 0,
-        GPL  = 1,
-        GPL_V2 = 1,
-        LGPL = 2,
-        LGPL_V2 = 2,
-        BSDL = 3,
-        Artistic = 4,
-        QPL = 5,
-        QPL_V1_0 = 5,
-        GPL_V3 = 6,
-        LGPL_V3 = 7,
-        LGPL_V2_1 = 8 ///< @since 5.25
+        Custom = -2, ///< Custom license
+        File = -1, ///< License set from text file, see setLicenseFromPath()
+        Unknown = 0, ///< Unknown license
+        GPL = 1, ///< GPL
+        GPL_V2 = GPL, ///< GPL_V2, this has the same value as LicenseKey::GPL, see https://spdx.org/licenses/GPL-2.0.html
+        LGPL = 2, ///< LGPL
+        LGPL_V2 = LGPL, ///< LGPL_V2, this has the same value as LicenseKey::LGPL, see https://spdx.org/licenses/LGPL-2.0-only.html
+        BSDL = 3, ///< BSDL, see https://spdx.org/licenses/BSD-2-Clause.html
+        Artistic = 4, ///< Artistic, see https://spdx.org/licenses/Artistic-2.0.html
+        QPL = 5, ///< QPL
+        QPL_V1_0 = QPL, ///< QPL_V1_0, this has the same value as LicenseKey::QPL, see https://spdx.org/licenses/QPL-1.0.html
+        GPL_V3 = 6, ///< GPL_V3, see https://spdx.org/licenses/GPL-3.0.html
+        LGPL_V3 = 7, ///< LGPL_V3, see https://spdx.org/licenses/LGPL-3.0-only.html
+        LGPL_V2_1 = 8, ///< LGPL_V2_1 @since 5.25, see https://spdx.org/licenses/LGPL-2.1-only.html
     };
     Q_ENUM(LicenseKey)
 
@@ -205,7 +214,7 @@ public:
      */
     enum NameFormat {
         ShortName,
-        FullName
+        FullName,
     };
     Q_ENUM(NameFormat)
 
@@ -214,7 +223,7 @@ public:
      */
     enum VersionRestriction {
         OnlyThisVersion,
-        OrLaterVersions
+        OrLaterVersions,
     };
     Q_ENUM(VersionRestriction)
 
@@ -306,9 +315,7 @@ private:
     /**
      * @internal Used by KAboutData to construct a predefined license.
      */
-    explicit KAboutLicense(enum KAboutLicense::LicenseKey licenseType,
-                           enum KAboutLicense::VersionRestriction versionRestriction,
-                           const KAboutData *aboutData);
+    explicit KAboutLicense(enum KAboutLicense::LicenseKey licenseType, enum KAboutLicense::VersionRestriction versionRestriction, const KAboutData *aboutData);
     /**
      * @internal Used by KAboutData to construct a predefined license.
      */
@@ -326,8 +333,141 @@ private:
      */
     void setLicenseFromText(const QString &licenseText);
 
-    class Private;
-    QSharedDataPointer<Private> d;
+private:
+    QSharedDataPointer<class KAboutLicensePrivate> d;
+};
+
+/**
+ * This class is used to store information about a third party component.
+ * It can store the component's name, a description, a link to a website
+ * and the license of the libary. This class is intended for use in the
+ * KAboutData class, but it can be used elsewhere as well.
+ * Normally you should at least define the libary's name.
+ * Creating a KAboutComponent object by yourself is relatively useless,
+ * but the KAboutData method KAboutData::libaries() return lists of
+ * KAboutComponent data objects which you can examine.
+ *
+ * Example usage within a main(), retrieving the list of components used
+ * by a program and re-using data from one of them:
+ *
+ * @code
+ * KAboutData about("khello", i18n("KHello"), "0.1",
+ *                   i18n("A KDE version of Hello, world!"),
+ *                   KAboutLicense::LGPL,
+ *                   i18n("Copyright (C) 2014 Developer"));
+ *
+ * about.addComponent(i18n("Awsom Lib"),
+ *                  i18n("Does awesom stuff. Copyright (C) 2014"),
+ *                  i18n("1.02.3"),
+ *                  "http://example.com",
+ *                  KAboutLicense::LGPL);
+ * QList<KAboutComponent> components = about.components();
+ * @endcode
+ *
+ * @since 5.84
+ */
+class KCOREADDONS_EXPORT KAboutComponent
+{
+    Q_GADGET
+    Q_PROPERTY(QString name READ name CONSTANT)
+    Q_PROPERTY(QString description READ description CONSTANT)
+    Q_PROPERTY(QString webAddress READ webAddress CONSTANT)
+    Q_PROPERTY(KAboutLicense licenses READ license CONSTANT)
+    Q_PROPERTY(QString version READ version CONSTANT)
+    friend class KAboutData;
+    friend class KAboutDataPrivate;
+
+public:
+    /**
+     * Convenience constructor
+     *
+     * @param name The name of the component.
+     *
+     * @param description The description of this component.
+     *
+     * @param version The version of this component.
+     *
+     * @param webAddress Website of the component.
+     *
+     * @param licenseType The license identifier of the component.
+     *
+     * @p name default argument
+     */
+    explicit KAboutComponent(const QString &name = QString(),
+                          const QString &description = QString(),
+                          const QString &version = QString(),
+                          const QString &webAddress = QString(),
+                          enum KAboutLicense::LicenseKey licenseType = KAboutLicense::Unknown);
+
+    /**
+     * Convenience constructor
+     *
+     * @param name The name of the component.
+     *
+     * @param description The description of this component.
+     *
+     * @param version The version of this component.
+     *
+     * @param webAddress Website of the component.
+     *
+     * @param pathToLicenseFile Path to the file in the local filesystem containing the license text.
+     *        The file format has to be plain text in an encoding compatible to the local.
+     *
+     * @p name default argument
+     */
+    explicit KAboutComponent(const QString &name,
+                          const QString &description,
+                          const QString &version,
+                          const QString &webAddress,
+                          const QString &pathToLicenseFile);
+
+    /**
+     * Copy constructor. Performs a deep copy.
+     * @param other object to copy
+     */
+    KAboutComponent(const KAboutComponent &other);
+
+    ~KAboutComponent();
+
+    /**
+     * Assignment operator. Performs a deep copy.
+     * @param other object to copy
+     */
+    KAboutComponent &operator=(const KAboutComponent &other);
+
+    /**
+     * The component's name
+     * @return the component's name (can be QString(), if it has been
+     *           constructed with an empty name)
+     */
+    QString name() const;
+
+    /**
+     * The component's description
+     * @return the component's description (can be empty)
+     */
+    QString description() const;
+
+    /**
+     * The component's version
+     * @return the component's task (can be empty)
+     */
+    QString version() const;
+
+    /**
+     * The website or a relevant link
+     * @return the component's website (can be empty)
+     */
+    QString webAddress() const;
+
+    /**
+     * The component's license
+     * @return the component's KAboutLicense
+     */
+    KAboutLicense license() const;
+
+private:
+    QSharedDataPointer<class KAboutComponentPrivate> d;
 };
 
 /**
@@ -403,14 +543,14 @@ class KCOREADDONS_EXPORT KAboutData
     Q_PROPERTY(QString bugAddress READ bugAddress CONSTANT)
     Q_PROPERTY(QString version READ version CONSTANT)
     Q_PROPERTY(QString otherText READ otherText CONSTANT)
-    Q_PROPERTY(QVariantList authors READ authorsVariant CONSTANT) //constant in practice as addAuthor is not exposed to Q_GADGET
+    Q_PROPERTY(QVariantList authors READ authorsVariant CONSTANT) // constant in practice as addAuthor is not exposed to Q_GADGET
     Q_PROPERTY(QVariantList credits READ creditsVariant CONSTANT)
     Q_PROPERTY(QVariantList translators READ translatorsVariant CONSTANT)
+    Q_PROPERTY(QVariantList components READ componentsVariant CONSTANT)
     Q_PROPERTY(QVariantList licenses READ licensesVariant CONSTANT)
     Q_PROPERTY(QString copyrightStatement READ copyrightStatement CONSTANT)
     Q_PROPERTY(QString desktopFileName READ desktopFileName CONSTANT)
 public:
-
     /**
      * Returns the KAboutData for the application.
      *
@@ -539,8 +679,7 @@ public:
                const QString &copyrightStatement = QString(),
                const QString &otherText = QString(),
                const QString &homePageAddress = QString(),
-               const QString &bugAddress = QStringLiteral("submit@bugs.kde.org")
-              );
+               const QString &bugAddress = QStringLiteral("submit@bugs.kde.org"));
 
     /**
      * Constructor.
@@ -560,10 +699,7 @@ public:
      *
      * @see setOrganizationDomain(const QByteArray&), setDesktopFileName(const QString&)
      */
-    explicit KAboutData(const QString &componentName = {},
-               const QString &displayName = {},
-               const QString &version = {}
-              );
+    explicit KAboutData(const QString &componentName = {}, const QString &displayName = {}, const QString &version = {});
 
     /**
      * Copy constructor.  Performs a deep copy.
@@ -659,8 +795,64 @@ public:
      * @param emailAddress the email address(es) of the translator(s)
      * @see KAboutTranslator
      */
-    KAboutData &setTranslator(const QString &name,
-                              const QString &emailAddress);
+    KAboutData &setTranslator(const QString &name, const QString &emailAddress);
+
+    /**
+     * Defines a component that is used by the application.
+     *
+     * You can call this function as many times as you need. Each entry is
+     * appended to a list.
+     *
+     * @param name The component's name. It should be translated.
+     *
+     * @param description Short description of the component and maybe
+     *        copyright info. This text can contain newlines. It should
+     *        be translated. Can be left empty.
+     *
+     * @param version The version of the component. Can be left empty.
+     *
+     * @param webAddress The component's homepage or a relevant link.
+     *        Start the address with "http://". "http://some.domain" is
+     *        correct, "some.domain" is not. Can be left empty.
+     *
+     * @param licenseKey The component's license identifier. Can be left empty (i.e. KAboutLicense::Unknown)
+     *
+     * @since 5.84
+     */
+    KAboutData &addComponent(const QString &name,
+                           const QString &description = QString(),
+                           const QString &version = QString(),
+                           const QString &webAddress = QString(),
+                           KAboutLicense::LicenseKey licenseKey = KAboutLicense::Unknown);
+
+    /**
+     * Defines a component that is used by the application with a custom license text file.
+     *
+     * You can call this function as many times as you need. Each entry is
+     * appended to a list.
+     *
+     * @param name The component's name. It should be translated.
+     *
+     * @param description Short description of the component and maybe
+     *        copyright info. This text can contain newlines. It should
+     *        be translated. Can be left empty.
+     *
+     * @param version The version of the component. Can be left empty.
+     *
+     * @param webAddress The component's homepage or a relevant link.
+     *        Start the address with "http://". "http://some.domain" is
+     *        correct, "some.domain" is not. Can be left empty.
+     *
+     * @param pathToLicenseFile Path to the file in the local filesystem containing the license text.
+     *        The file format has to be plain text in an encoding compatible to the local.
+     *
+     * @since 5.84
+     */
+    KAboutData &addComponent(const QString &name,
+                           const QString &description,
+                           const QString &version,
+                           const QString &webAddress,
+                           const QString &pathToLicenseFile);
 
     /**
      * Defines a license text, which is translated.
@@ -756,7 +948,7 @@ public:
      *
      * @param image logo image.
      * @see programLogo()
-    */
+     */
     KAboutData &setProgramLogo(const QVariant &image);
 
     /**
@@ -807,8 +999,7 @@ public:
      *
      * @since 5.37
      */
-    KAboutData &setLicense(KAboutLicense::LicenseKey licenseKey,
-                           KAboutLicense::VersionRestriction versionRestriction);
+    KAboutData &setLicense(KAboutLicense::LicenseKey licenseKey, KAboutLicense::VersionRestriction versionRestriction);
 
     /**
      * Adds a license identifier.
@@ -834,8 +1025,7 @@ public:
      *
      * @since 5.37
      */
-    KAboutData &addLicense(KAboutLicense::LicenseKey licenseKey,
-                           KAboutLicense::VersionRestriction versionRestriction);
+    KAboutData &addLicense(KAboutLicense::LicenseKey licenseKey, KAboutLicense::VersionRestriction versionRestriction);
 
     /**
      * Defines the copyright statement to show when displaying the license.
@@ -929,6 +1119,12 @@ public:
      */
     QString productName() const;
 
+    /**
+     * @internal
+     * Provided for use by KCrash
+     */
+    const char *internalProductName() const;
+
     /**
      * Returns the translated program name.
      * @return the program name (translated).
@@ -957,8 +1153,9 @@ public:
      * @return the program's icon name.
      *
      * This is mostly for compatibility, given that setProgramIconName is deprecated.
+     * @deprecated Since 5.2, Use QGuiApplication::windowIcon instead
      */
-    KCOREADDONS_DEPRECATED_VERSION(5, 2, "Use QApplication::windowIcon")
+    KCOREADDONS_DEPRECATED_VERSION(5, 2, "Use QGuiApplication::windowIcon instead")
     QString programIconName() const;
 #endif
 
@@ -1041,6 +1238,13 @@ public:
      */
     static QString aboutTranslationTeam();
 
+    /**
+     * Returns a list of components.
+     * @return component information (list of components).
+     * @since 5.84
+     */
+    QList<KAboutComponent> components() const;
+
     /**
      * Returns a translated, free form text.
      * @return the free form text (translated). Can be QString() if not set.
@@ -1098,8 +1302,7 @@ public:
      * displayed at all.  Call unsetCustomAuthorText() to revert to the default
      * message.
      */
-    KAboutData &setCustomAuthorText(const QString &plainText,
-                                    const QString &richText);
+    KAboutData &setCustomAuthorText(const QString &plainText, const QString &richText);
 
     /**
      * Clears any custom text displayed around the list of authors and falls
@@ -1174,17 +1377,18 @@ private:
     QVariantList authorsVariant() const;
     QVariantList creditsVariant() const;
     QVariantList translatorsVariant() const;
+    QVariantList componentsVariant() const;
 
     friend void KCrash::defaultCrashHandler(int sig);
     static const KAboutData *applicationDataPointer();
 
-    class Private;
-    Private *const d;
+private:
+    std::unique_ptr<class KAboutDataPrivate> const d;
 };
 
 Q_DECLARE_METATYPE(KAboutData)
 Q_DECLARE_METATYPE(KAboutLicense)
 Q_DECLARE_METATYPE(KAboutPerson)
+Q_DECLARE_METATYPE(KAboutComponent)
 
 #endif
-
